//
// DataLayout.java
//
// This is the base class for all
// layout plugins...
//

package spg;

import java.awt.*;

public abstract class DataLayout {
    final protected String name;
    protected Rectangle bounds;
    protected double greatestValue, leastValue;
    protected DataGlyph currentGlyphType;
    protected Font labelFont;
    protected FontMetrics labelFM;

    protected int numOptions;
    protected boolean optionFlags[];
    protected String optionNames[];

    public DataLayout(String myName) {
	name = myName;
	bounds = new Rectangle(0, 0, 0, 0);
    }

    public String getName() {
	return name;
    }

    public int getNumOptions() {
	return numOptions;
    }

    public String getOptionName(int index) {
	if ((index >= 0) && (index < numOptions))
	    return optionNames[index];
	else
	    return null;
    }

    public boolean getOptionValue(int index) {
	if ((index >= 0) && (index < numOptions))
	    return optionFlags[index];
	else
	    return false;
    }

    public void setOptionValue(int index, boolean value) {
	if ((index >= 0) && (index < numOptions))
	    optionFlags[index] = value;
    }

    public Rectangle getLayoutBounds(Dimension d) {
	return bounds;
    }

    public void setLabelFontInfo(Font f, FontMetrics fm) {
	labelFont = f;
	labelFM = fm;
    }

    public void drawLayout(Graphics g, ViewerState global) {
    }

    public void drawDataDump(Graphics g, int x, int y, int rightBoundary, int bottomBoundary,
	int dataPointIndex, ViewerState global)
    {
	// This code is shared by all
	// layout types for drawing a
	// datum's raw values...
	int rectWidth, rectHeight;
	double dataPoint[];

	// Get original raw data point
	dataPoint = global.constant.data[dataPointIndex];
	
	// Figure out the size of the data text
	rectHeight = 10 + ((global.current.enabledDim + 1) * labelFM.getHeight());
	rectWidth = labelFM.stringWidth("" + dataPoint[global.current.dimMap[0]] + ":");
	for (int i = 1; i <= global.current.enabledDim; i++) {
	    int testWidth = labelFM.stringWidth("" + dataPoint[global.current.dimMap[i]]);
	    if (testWidth > rectWidth)
		rectWidth = testWidth;
	}
	rectWidth += 10;    // Padding

	// Push in rectangle if it would fall outside of canvas
	if ((x + rectWidth) >= rightBoundary)
	    x -= (x + rectWidth - rightBoundary + 1);
	if ((y + rectHeight) >= bottomBoundary)
	    y -= (y + rectHeight - bottomBoundary + 1);

	// Clear a rectangular region for the data
	g.setColor(new Color(251, 222, 136));	    // Light-orange
	g.fillRect(x, y, rectWidth, rectHeight);

	// Draw each dimension
	g.setColor(Color.black);
	g.setFont(labelFont);
	g.drawString("" + dataPoint[global.current.dimMap[0]] + ":", x + 5, 
	    y + 5 + labelFM.getAscent());
	for (int i = 1; i <= global.current.enabledDim; i++) {
	    g.drawString("" + dataPoint[global.current.dimMap[i]], x + 5,
		y + 5 + labelFM.getAscent() + (i * labelFM.getHeight()));
	}
    }
}