// Viewer.java (v1.4)
//
// Source for SpiralGlyphics, the demo accompanying the paper
// "Visualization of Cyclic Multivariate Data," appearing in
// the Late Breaking Hot Topics section of the proceeedings of
// IEEE Visualization Ninety-Seven.
//
// Copyright (c) 1997, 1998 by Benjamin Nason Lipchak,
//                             Compaq Computer Corporation
//
// DISCLAIMER:
//      This code is not pretty, well-documented, or guaranteed
//      not to explode your system.  Neither I nor Compaq Computer
//      Corporation will accept any responsibility for problems
//      associated with the use of this software.
//
// Look at http://www.cs.wpi.edu/~matt/research/Cyclic/
// for the latest versions and information about SpiralGlyphics,
// or send email to matt@cs.wpi.edu or Benjamin.Lipchak@digital.com.

package spg;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;
import java.util.*;
import java.net.*;

import spg.plugins.*;

public class Viewer extends Applet implements WindowListener, ActionListener,
							ComponentListener, Runnable {
    ViewerWorkSpace workSpace;
    ViewerState global;
    private boolean inAnApplet = true;
    private String dataFilePath = null;
    private Button launchButton;
    private Thread loadThread = null;

    private boolean GUICreated = false;
    private boolean windowOpen = false;
    private boolean appletGetImageWorks = true;

    MediaTracker tracker;

    public void init() {
	global = new ViewerState();

	tracker = new MediaTracker(this);
	if (inAnApplet) {
//System.err.println("java.vendor == " + System.getProperty("java.vendor"));
//String javaVendor = System.getProperty("java.vendor");
	    if (appletGetImageWorks) {
		global.titleImage = getImage(getCodeBase(), "spg/spg.gif");
	    } else {
		// Don't ask me why the above returns null with some VMs!!
		// Use Toolkit version instead.
		try {
		    global.titleImage = Toolkit.getDefaultToolkit().getImage(
			new URL(getCodeBase() + "spg/spg.gif"));
		} catch (MalformedURLException e) {
		}
	    }
	    if (global.titleImage == null) {
		System.err.println("Corrupt distribution!  Unable to load title image!");
		destroy();
		return;
	    }
	    tracker.addImage(global.titleImage, 0);
	    global.inAnApplet = true;

	    String buttonString, fgColor, bgColor;

	    buttonString = getParameter("BUTTONTEXT");
	    dataFilePath = getParameter("DATAFILE");
	    fgColor = getParameter("FGCOLOR");
	    bgColor = getParameter("BGCOLOR");

	    try {
		Color fgc = Color.decode(fgColor);
		setForeground(fgc);
	    } catch (NumberFormatException e) {
	    }
	    try {
		Color bgc = Color.decode(bgColor);
		setBackground(bgc);
	    } catch (NumberFormatException e) {
	    }

	    launchButton = new Button();
	    launchButton.addActionListener(this);

	    // Query for available layout and glyph plugins...
	    if (!getPlugins()) {
		buttonString = "Unable to load plugins!";
		launchButton.setEnabled(false);
	    } else if (dataFilePath == null) {
		buttonString = "No data set specified!";
		launchButton.setEnabled(false);
	    }

	    if (buttonString == null)
		buttonString = "SpiralGlyphics: " + dataFilePath;
	    launchButton.setLabel(buttonString);
	    add(launchButton);

	    validate();
	} else {
	    global.titleImage = Toolkit.getDefaultToolkit().getImage("spg/spg.gif");
	    if (global.titleImage == null) {
		System.err.println("Corrupt distribution!  Unable to load title image!");
		System.exit(1);
		return;
	    }
	    tracker.addImage(global.titleImage, 0);
	    global.inAnApplet = false;

	    // Query for available layout and glyph plugins...
	    getPlugins();

	    // Wait for title image to load
	    boolean imageLoaded = false;
	    while (!imageLoaded) {
		try {
		    tracker.waitForAll();
		    imageLoaded = true;
		} catch (InterruptedException e) {
		}
	    }

	    initViewerContent(this);
	}
    }

    public void start() {
	if (!inAnApplet || GUICreated) {
	    global.sideControls.setEnabled(true);
	    workSpace.setEnabled(true);
	}
    }

    public void stop() {
	loadThread = null;
	if (!inAnApplet || GUICreated) {
	    global.sideControls.setEnabled(false);
	    workSpace.setEnabled(false);
	}
    }

    public void destroy() {
	removeAll();
	if (GUICreated) {
	    global.frame.dispose();
	}
    }

    public void run() {
	// Wait for title image to load
	boolean imageLoaded = false;
	while (!imageLoaded) {
	    try {
		tracker.waitForAll();
		imageLoaded = true;
	    } catch (InterruptedException e) {
	    }
	}

	// Create GUI components
	Dimension screenSize;
	int desiredWidth, desiredHeight;

	Panel newPanel = new Panel();
	initViewerContent(newPanel);

	global.frame = 
	    new ViewerFrame("SpiralGlyphics: Cyclic Glyph Viewer", global);
	global.frame.add("Center", newPanel);
	screenSize = global.frame.getToolkit().getScreenSize();
	desiredWidth = screenSize.width * 5 / 6;
	desiredHeight = screenSize.height * 5 / 6;
	global.frame.setBounds((screenSize.width - desiredWidth) / 2, 
	    (screenSize.height - desiredHeight) / 2, 
	    desiredWidth, desiredHeight);
	global.frame.validate();
	global.frame.addWindowListener(this);
	global.frame.addComponentListener(this);
	// Since we're reusing this context, we need to invalidate the layout
	global.layoutTypeDirty = true; 
	global.frame.show();

	GUICreated = true;

	// Now we wait for the windowOpen flag so we can
	// load the data set.
	while (!windowOpen) {
	    try {
		Thread.sleep(1000);
	    } catch (InterruptedException e){
		// the VM doesn't want us to sleep anymore,
		// so get back to work
	    }
	}

	// Now load data set
	global.loadState(getCodeBase() + dataFilePath);
		
	// Redraw both canvases
	global.canvas.repaint();
	global.sideControls.glyphLegendCanvas.repaint();

	global.sideControls.setEnabled(true);
	workSpace.setEnabled(true);

	loadThread = null;
    }

    public void actionPerformed(ActionEvent event) {
	launchButton.setEnabled(false);

	if (dataFilePath != null) {
	    loadThread = new Thread(this, "Load Thread");
	    loadThread.start();
	}    
    }

    public void windowClosed(WindowEvent event) {
	GUICreated = false;
	if (inAnApplet) {
	    // Re-enable launchButton
	    launchButton.setEnabled(true);
	}
    }

    public void windowDeiconified(WindowEvent event) {
    }

    public void windowIconified(WindowEvent event) {
    }

    public void windowActivated(WindowEvent event) {
    }

    public void windowDeactivated(WindowEvent event) {
    }

    public void windowOpened(WindowEvent event) {
	windowOpen = true;
    }

    public void windowClosing(WindowEvent event) {
	if (inAnApplet) {
	    global.frame.dispose();
	} else {
	    // Check if we need to save
	    if (global.checkForSave())
		System.exit(0);
	}
	windowOpen = false;
    }

    public void componentHidden(ComponentEvent event) {
    }

    public void componentMoved(ComponentEvent event) {
    }

    public void componentResized(ComponentEvent event) {
	if (!inAnApplet) {
	    global.frame.validate();
	}
    }

    public void componentShown(ComponentEvent event) {
    }

    public void initViewerContent(Panel panel) {
	GridBagLayout gridBag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	panel.setLayout(gridBag);

	c.fill = GridBagConstraints.BOTH;
	c.weightx = 0.1;
	global.sideControls = new ViewerSideControls(global);
	gridBag.setConstraints(global.sideControls, c);
	panel.add(global.sideControls);

	c.weightx = 0.9;
	c.weighty = 1.0;
	c.gridx = 1;
	workSpace = new ViewerWorkSpace(global);
	gridBag.setConstraints(workSpace, c);
	panel.add(workSpace);

	panel.validate();
    }

    public static void main(String args[]) {
	Viewer viewer = new Viewer();
	ViewerState global;
	Dimension screenSize;
	int desiredWidth, desiredHeight;

	viewer.inAnApplet = false;

	viewer.init();
	viewer.start();

	global = viewer.global;
	global.frame = 
	    new ViewerFrame("SpiralGlyphics: Cyclic Glyph Viewer", global);
	global.frame.add("Center", viewer);
	screenSize = global.frame.getToolkit().getScreenSize();
	desiredWidth = screenSize.width * 5 / 6;
	desiredHeight = screenSize.height * 5 / 6;
	global.frame.setBounds((screenSize.width - desiredWidth) / 2, 
	    (screenSize.height - desiredHeight) / 2, 
	    desiredWidth, desiredHeight);
	global.frame.validate();
	global.frame.addWindowListener(viewer);
	global.frame.addComponentListener(viewer);
	global.frame.show();
    }

    boolean getPlugins() {
	String[] candidateClasses = null;
	FilenameFilter filter;
	File lister;
	int numLayouts = 0, numGlyphs = 0;
	DataLayout[] tempLayouts = null;
	DataGlyph[] tempGlyphs = null;
	boolean fatalPluginError = false;

	if (inAnApplet) {
	    // A list of available plug-ins must be
	    // included in the applet's PARAM tags.

	    String layoutPluginParams = getParameter("LAYOUTPLUGINS");
	    if (layoutPluginParams != null) {
		StringTokenizer st = new StringTokenizer(layoutPluginParams);
		int numTokens = st.countTokens();
		candidateClasses = new String[numTokens];
		int i = 0;
		while ((numTokens--) > 0) {
		    candidateClasses[i++] = st.nextToken() + "Plugin.class";
		}
	    }
	} else {
	    // Look for classes in the plugins directory,
	    // enumerate them, and load them.
	    //
	    // Look for files with names *LayoutPlugin.class
	    // and *GlyphPlugin.class in the ./plugins
	    // directory.

	    filter = new LayoutPluginFilenameFilter();
	    lister = new File("./spg/plugins");

	    try {
		candidateClasses = lister.list(filter);
	    } catch (SecurityException e) {
		// Ignore these exceptions...
	    }
	}

	if ((candidateClasses == null) || (candidateClasses.length == 0)) {
	    fatalPluginError = true;
	} else {
	    tempLayouts = new DataLayout[candidateClasses.length];
	    numLayouts = 0;
	    for (int i = 0; i < candidateClasses.length; i++) {
		candidateClasses[i] = "spg.plugins." + candidateClasses[i].substring(0, candidateClasses[i].length() - 6);
		try {
		    tempLayouts[numLayouts] = (DataLayout)Class.forName(candidateClasses[i]).newInstance();
		    numLayouts++;
		} catch (Exception e) {
		    // Ignore... Corrupted or read-only file, probably...
		    System.err.println("Exception when trying to load layout plugin " + 
			candidateClasses[i] + ": " + e);
		}
	    }
	    if (numLayouts < 1) {
		fatalPluginError = true;
	    }
	}

	if (inAnApplet) {
	    // A list of available plug-ins must be
	    // included in the applet's PARAM tags.

	    String glyphPluginParams = getParameter("GLYPHPLUGINS");
	    if (glyphPluginParams != null) {
		StringTokenizer st = new StringTokenizer(glyphPluginParams);
		int numTokens = st.countTokens();
		candidateClasses = new String[numTokens];
		int i = 0;
		while ((numTokens--) > 0) {
		    candidateClasses[i++] = st.nextToken() + "Plugin.class";
		}
	    }
	} else {
	    // Look for classes in the plugins directory,
	    // enumerate them, and load them.
	    //
	    // Look for files with names *LayoutPlugin.class
	    // and *GlyphPlugin.class in the ./plugins
	    // directory.

	    filter = new GlyphPluginFilenameFilter();
	    lister = new File("./spg/plugins");

	    try {
		candidateClasses = lister.list(filter);
	    } catch (SecurityException e) {
		// Ignore these exceptions...
	    }
	}

	if ((candidateClasses == null) || (candidateClasses.length == 0)) {
	    fatalPluginError = true;
	} else {
	    tempGlyphs = new DataGlyph[candidateClasses.length];
	    numGlyphs = 0;
	    for (int i = 0; i < candidateClasses.length; i++) {
		candidateClasses[i] = "spg.plugins." + candidateClasses[i].substring(0, candidateClasses[i].length() - 6);
		try {
		    tempGlyphs[numGlyphs] = (DataGlyph)Class.forName(candidateClasses[i]).newInstance();
		    numGlyphs++;
		} catch (Exception e) {
		    // Ignore... Corrupted or read-only file, probably...
		    System.err.println("Exception when trying to load glyph plugin " + 
			candidateClasses[i] + ": " + e);
		}
	    }
	    if (numGlyphs < 1) {
		fatalPluginError = true;
	    }
	}

	if (fatalPluginError) {
	    System.err.println("");
	    System.err.println("Error loading plugins!");
	    System.err.println("");
	    System.err.println("At least one layout plugin and one glyph plugin");
	    System.err.println("must be provided in the plugins directory!");
	    System.err.println("");

	    // If we're in an applet, we'll display an error message instead.
	    if (inAnApplet) {
		return false;
	    } else {
		System.err.println("Aborting!");
		System.exit(1);
	    }
	} else {
	    global.numLayoutPlugins = numLayouts;
	    global.layoutPlugins = new DataLayout[numLayouts];
	    for (int i = 0; i < numLayouts; i++) {
		global.layoutPlugins[i] = tempLayouts[i];
		System.out.println("Loading " + global.layoutPlugins[i].getName() + " plugin...");
	    }

	    global.numGlyphPlugins = numGlyphs;
	    global.glyphPlugins = new DataGlyph[numGlyphs];
	    for (int i = 0; i < numGlyphs; i++) {
		global.glyphPlugins[i] = tempGlyphs[i];
		System.out.println("Loading " + global.glyphPlugins[i].getName() + " plugin...");
	    }
	}

	return true;
    }

    public String getAppletInfo() {
	return "SpiralGlyphics: Cyclic Multivariate Data Viewer";
    }

    public String[][] getParameterInfo() {
	String[][] info = {
	    // Parameter Name   Kind of Value   Description
	    {"LAYOUTPLUGINS",   "String",	    "REQUIRED: class names (whitespace delimited, omit Plugin.class)"},
	    {"GLYPHPLUGINS",    "String",	    "REQUIRED: class names (whitespace delimited, omit Plugin.class)"},
	    {"DATAFILE",	"String",	    "REQUIRED: data set (path to .spg file)"},
	    {"BUTTONTEXT",	"String",	    "OPTIONAL: text to appear on launch button"},
	    {"FGCOLOR",		"Color",	    "OPTIONAL: color for button text in RGB hex 0x****** form"},
	    {"BGCOLOR",		"Color",	    "OPTIONAL: color for button in RGB hex 0x****** form"}
	};

	return info;
    }
}

class LayoutPluginFilenameFilter implements FilenameFilter {
    public boolean accept(File dir, String name) {
	return name.toLowerCase().endsWith("layoutplugin.class");
    }
}

class GlyphPluginFilenameFilter implements FilenameFilter {
    public boolean accept(File dir, String name) {
	return name.toLowerCase().endsWith("glyphplugin.class");
    }
}

class ViewerWorkSpace extends Panel {
    ViewerState global;

    public ViewerWorkSpace(ViewerState gInfo) {
	global = gInfo;

	gInfo.canvas = new ViewerCanvas(gInfo);
	gInfo.bottomControls = new ViewerBottomControls(gInfo);

	setLayout(new BorderLayout());
	add("Center", gInfo.canvas);
	add("South", gInfo.bottomControls);

	validate();
    }
}
