//
// ViewerBottomControls.java
//
// These are the GUI controls that appear
// under the data window...
//

package spg;

import java.awt.*;
import java.awt.event.*;

public class ViewerBottomControls extends Panel 
			implements AdjustmentListener {
    Label tMin, tMax, tValLabel, tLabel;
    Scrollbar tScrollbar;
    ViewerState global;

    public ViewerBottomControls(ViewerState gInfo) {
	global = gInfo;

	GridBagLayout gridBag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gridBag);

	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = 0;
	c.gridy = 0;
	c.weightx = 1.0;

	c.gridwidth = 1;
	tMin = new Label("" + global.constant.tMin);
	tValLabel = 
	   new Label("[" + global.current.tVal + ", " + 
	   global.derived.tVal2 + "]", 
	   Label.CENTER);
	tMax = new Label("" + global.constant.tMax, Label.RIGHT);
	c.insets = new Insets(5,10,0,0);
	gridBag.setConstraints(tMin, c);
	c.gridx = 1;
	c.insets = new Insets(5,0,0,0);
	gridBag.setConstraints(tValLabel, c);
	c.gridx = 2;
	c.insets = new Insets(5,0,0,10);
	gridBag.setConstraints(tMax, c);
	c.gridwidth = 3;
	c.gridx = 0;
	c.gridy = 2;
	c.insets = new Insets(0,10,10,10);
	tLabel = new Label("Visible Interval", Label.CENTER);
	gridBag.setConstraints(tLabel, c);

	c.gridy = 1;
	c.insets = new Insets(0,10,0,10);
	int tMaxVal = 
	   (int)Math.rint((global.constant.tMax - global.derived.tInterval + 
		global.constant.tMinimumUnit) * 
	   ViewerState.TSCROLLFACTOR);
	int tShow = (int)Math.rint(global.derived.tInterval * ViewerState.TSCROLLFACTOR);
	tScrollbar = new Scrollbar(Scrollbar.HORIZONTAL,
	  (int)Math.rint(global.current.tVal * ViewerState.TSCROLLFACTOR), 
	  tShow, 
	  (int)Math.rint(global.constant.tMin * ViewerState.TSCROLLFACTOR), 
	  tMaxVal + tShow); 
	tScrollbar.setBlockIncrement(
	   (int)Math.rint(global.current.cycleLength * ViewerState.TSCROLLFACTOR));
	tScrollbar.setEnabled(false);
	tScrollbar.addAdjustmentListener(this);
	gridBag.setConstraints(tScrollbar, c);

	add(tMin); add(tValLabel); add(tMax);
	add(tScrollbar);
	add(tLabel);

	validate();
    }

    public synchronized void adjustmentValueChanged(AdjustmentEvent ev) {
	Adjustable whichOne = ev.getAdjustable();
	if (whichOne == tScrollbar) {
	   // Beginning of visible interval
	   global.current.tVal = ViewerState.massageDouble(
		(double)tScrollbar.getValue() / 
		ViewerState.TSCROLLFACTOR, ViewerState.TSCROLLFACTOR);

	   // End of visible interval
	   global.derived.tVal2 = ViewerState.massageDouble(global.current.tVal + 
		global.derived.tInterval, ViewerState.TSCROLLFACTOR);

	   tValLabel.setText("[" + global.current.tVal + ", " + 
	      global.derived.tVal2 + "]"); 

	   global.canvas.repaint();	// THROTTLE HERE?
	} 
    }
}

