//
// ViewerFrame.java
//
// Here lies the code associated with the
// menu bar...
//

package spg;

import java.awt.*;
import java.awt.event.*;

public class ViewerFrame extends Frame implements ItemListener,
						ActionListener {
    ViewerState global;

    MenuBar mb;
    Menu fileMenu, glyphMenu, layoutMenu;
    Menu optionsMenu, helpMenu;

    MenuItem openMenuItem, importMenuItem;
    MenuItem saveMenuItem, saveAsMenuItem, exitMenuItem;

    Menu glyphTypeMenu;
    CheckboxMenuItem glyphCheckboxMenuItems[];

    Menu layoutTypeMenu;
    CheckboxMenuItem layoutCheckboxMenuItems[];

    CheckboxMenuItem glyphSpecificCheckboxMenuItems[][];
    CheckboxMenuItem layoutSpecificCheckboxMenuItems[][];
    
    CheckboxMenuItem autoNormCheckboxMenuItem;
    CheckboxMenuItem labelsCheckboxMenuItem;
    CheckboxMenuItem guidelinesCheckboxMenuItem;
    CheckboxMenuItem fillGlyphsCheckboxMenuItem;
    Menu colorUsageMenu;
    CheckboxMenuItem colorPerDimCheckboxMenuItem;
    CheckboxMenuItem colorPerGlyphCheckboxMenuItem;

    MenuItem javaInfoMenuItem;
    MenuItem aboutMenuItem;

    public ViewerFrame(String title, ViewerState gInfo) {
	super(title);
	global = gInfo;

	mb = new MenuBar();
	setMenuBar(mb);

	layoutMenu = new Menu("Layout");
	layoutMenu.setEnabled(false);
	glyphMenu = new Menu("Glyph");
	glyphMenu.setEnabled(false);
	optionsMenu = new Menu("Options");
	optionsMenu.setEnabled(false);
	helpMenu = new Menu("Help");

	if (!global.inAnApplet) {
	    fileMenu = new Menu("File");
	    mb.add(fileMenu);

	    openMenuItem = new MenuItem("Open...");
	    importMenuItem = new MenuItem("Import...");
	    saveMenuItem = new MenuItem("Save");
	    saveMenuItem.setEnabled(false);
	    saveAsMenuItem = new MenuItem("Save As...");
	    saveAsMenuItem.setEnabled(false);
	    exitMenuItem = new MenuItem("Exit");
	    openMenuItem.addActionListener(this);
	    importMenuItem.addActionListener(this);
	    saveMenuItem.addActionListener(this);
	    saveAsMenuItem.addActionListener(this);
	    exitMenuItem.addActionListener(this);
	    fileMenu.add(openMenuItem);
	    fileMenu.add(importMenuItem);
	    fileMenu.addSeparator();
	    fileMenu.add(saveMenuItem);
	    fileMenu.add(saveAsMenuItem);
	    fileMenu.addSeparator();
	    fileMenu.add(exitMenuItem);
	}

	mb.add(layoutMenu);
	mb.add(glyphMenu);
	mb.add(optionsMenu);
	mb.add(helpMenu);
	mb.setHelpMenu(helpMenu);

	layoutTypeMenu = new Menu("Layout Type  ");
	layoutCheckboxMenuItems = new CheckboxMenuItem[global.numLayoutPlugins];
	for (int i = 0; i < global.numLayoutPlugins; i++) {
	    layoutCheckboxMenuItems[i] = new CheckboxMenuItem(global.layoutPlugins[i].getName(),
		(global.current.layoutType == i));
	    layoutCheckboxMenuItems[i].addItemListener(this);
	}
	
	// Layout specific options
	layoutSpecificCheckboxMenuItems = new CheckboxMenuItem[global.numLayoutPlugins][];
	for (int i = 0; i < global.numLayoutPlugins; i++) {
	    int numOptions = global.layoutPlugins[i].getNumOptions();
	    layoutSpecificCheckboxMenuItems[i] = new CheckboxMenuItem[numOptions];
	    for (int j = 0; j < numOptions; j++) {
		layoutSpecificCheckboxMenuItems[i][j] = 
		    new CheckboxMenuItem(global.layoutPlugins[i].getOptionName(j),
		    global.layoutPlugins[i].getOptionValue(j));
		layoutSpecificCheckboxMenuItems[i][j].setEnabled(
		    (global.current.layoutType == i));
		layoutSpecificCheckboxMenuItems[i][j].addItemListener(this);
	    }
	}

	layoutMenu.add(layoutTypeMenu);
	for (int i = 0; i < global.numGlyphPlugins; i++) {
	    layoutTypeMenu.add(layoutCheckboxMenuItems[i]);
	}

	for (int i = 0; i < global.numLayoutPlugins; i++) {
	    int numOptions = global.layoutPlugins[i].getNumOptions();
	    if (numOptions > 0) {
		layoutMenu.addSeparator();
		layoutMenu.add("" + global.layoutPlugins[i].getName() + ":");
		for (int j = 0;  j < numOptions; j++) {
		    layoutMenu.add(layoutSpecificCheckboxMenuItems[i][j]);
		}
	    }
	}

	glyphTypeMenu = new Menu("Glyph Type  "); 
	glyphCheckboxMenuItems = new CheckboxMenuItem[global.numGlyphPlugins];
	for (int i = 0; i < global.numGlyphPlugins; i++) {
	    glyphCheckboxMenuItems[i] = new CheckboxMenuItem(global.glyphPlugins[i].getName(),
		(global.current.glyphType == i));
	    glyphCheckboxMenuItems[i].addItemListener(this);
	}
	
	// Glyph specific options
	glyphSpecificCheckboxMenuItems = new CheckboxMenuItem[global.numGlyphPlugins][];
	for (int i = 0; i < global.numGlyphPlugins; i++) {
	    int numOptions = global.glyphPlugins[i].getNumOptions();
	    glyphSpecificCheckboxMenuItems[i] = new CheckboxMenuItem[numOptions];
	    for (int j = 0; j < numOptions; j++) {
		glyphSpecificCheckboxMenuItems[i][j] = 
		    new CheckboxMenuItem(global.glyphPlugins[i].getOptionName(j),
		    global.glyphPlugins[i].getOptionValue(j));
		glyphSpecificCheckboxMenuItems[i][j].setEnabled(
		    (global.current.glyphType == i));
		glyphSpecificCheckboxMenuItems[i][j].addItemListener(this);
	    }
	}

	glyphMenu.add(glyphTypeMenu);
	for (int i = 0; i < global.numGlyphPlugins; i++) {
	    glyphTypeMenu.add(glyphCheckboxMenuItems[i]);
	}

	for (int i = 0; i < global.numGlyphPlugins; i++) {
	    int numOptions = global.glyphPlugins[i].getNumOptions();
	    if (numOptions > 0) {
		glyphMenu.addSeparator();
		glyphMenu.add("" + global.glyphPlugins[i].getName() + ":");
		for (int j = 0;  j < numOptions; j++) {
		    glyphMenu.add(glyphSpecificCheckboxMenuItems[i][j]);
		}
	    }
	}

	autoNormCheckboxMenuItem = new CheckboxMenuItem("Auto Normalize",
	    global.current.autoNormalize);
	labelsCheckboxMenuItem = new CheckboxMenuItem("Show Labels",
	    global.current.showLabels);
	guidelinesCheckboxMenuItem = 
	    new CheckboxMenuItem("Show Guidelines", 
	    global.current.showGuidelines);
	fillGlyphsCheckboxMenuItem = 
	    new CheckboxMenuItem("Fill Glyphs", 
	    global.current.fillGlyphs);
	colorUsageMenu = new Menu("Color Usage  ");
	colorPerDimCheckboxMenuItem = new CheckboxMenuItem("Color Per Dimension",
	    !global.current.colorPerGlyph);
	colorPerGlyphCheckboxMenuItem = new CheckboxMenuItem("Color Per Glyph",
	    global.current.colorPerGlyph);
	autoNormCheckboxMenuItem.addItemListener(this);
	labelsCheckboxMenuItem.addItemListener(this);
	guidelinesCheckboxMenuItem.addItemListener(this);
	fillGlyphsCheckboxMenuItem.addItemListener(this);
	colorPerDimCheckboxMenuItem.addItemListener(this);
	colorPerGlyphCheckboxMenuItem.addItemListener(this);
	optionsMenu.add(autoNormCheckboxMenuItem);
	optionsMenu.add(labelsCheckboxMenuItem);
	optionsMenu.add(guidelinesCheckboxMenuItem);
	optionsMenu.add(fillGlyphsCheckboxMenuItem);
	optionsMenu.add(colorUsageMenu);
	colorUsageMenu.add(colorPerDimCheckboxMenuItem);
	colorUsageMenu.add(colorPerGlyphCheckboxMenuItem);

	javaInfoMenuItem = new MenuItem("About your Java VM");
	javaInfoMenuItem.addActionListener(this);
	helpMenu.add(javaInfoMenuItem);
	aboutMenuItem = new MenuItem("About this Viewer");
	aboutMenuItem.addActionListener(this);
	helpMenu.add(aboutMenuItem);
    }

    public synchronized void actionPerformed(ActionEvent ev) {
	Object whichOne = ev.getSource();
	if (whichOne == exitMenuItem) {
	    // Check if we need to save
	    if (global.checkForSave())
		System.exit(0);
	} else if (whichOne == openMenuItem) {
	    // Check if we need to save
	    if (!global.checkForSave())
		return;

	    // Get name from dialog
	    Dimension d;
	    Rectangle r;

	    MyFileDialog openDialog = new MyFileDialog(this, "Open...", 
		FileDialog.LOAD, global);
	    r = getBounds();
	    d = openDialog.getSize();
	    openDialog.setLocation((r.width - d.width) / 2 + r.x, 
		(r.height - d.height) / 2 + r.y);
	    openDialog.pack();
	    openDialog.show();

	    this.toFront();

	    String newName = openDialog.getFile();
	    if (newName != null) {
		String newDirectory = openDialog.getDirectory();
		if (newDirectory != null) {
		    newName = newDirectory + newName;
		}
		global.loadState(newName);
	    }
		
	    // Redraw both canvases
	    global.canvas.repaint();
	    global.sideControls.glyphLegendCanvas.repaint();
	} else if (whichOne == saveMenuItem) {
	    // Save using current fileName, no dialogs
	    global.saveState();
	} else if (whichOne == saveAsMenuItem) {
	    // Get name from dialog
	    Dimension d;
	    Rectangle r;

	    MyFileDialog saDialog = new MyFileDialog(this, "Save As...", 
		FileDialog.SAVE, global);
	    r = getBounds();
	    d = saDialog.getSize();
	    saDialog.setLocation((r.width - d.width) / 2 + r.x, 
		(r.height - d.height) / 2 + r.y);
	    saDialog.pack();
	    saDialog.show();

	    this.toFront();

	    String newName = saDialog.getFile();
	    if (newName != null) {
		String newDirectory = saDialog.getDirectory();
		if (newDirectory != null) {
		    newName = newDirectory + newName;
		}
		global.fileName = newName;
		global.frame.setTitle("SpiralGlyphics - " + newName);
		global.saveState();
	    }
	} else if (whichOne == javaInfoMenuItem) {
	    // Bring up Java VM info dialog
	    Dimension d;
	    Rectangle r;
	    String message[] = new String[14];
	    int just[] = new int[14];

	    try {
		message[0] = "  java.version: " + System.getProperty("java.version");
	    } catch (SecurityException e) {
		message[0] = "  java.version: INACCESSIBLE";
	    }
	    try {
		message[1] = "  java.vendor: " + System.getProperty("java.vendor");
	    } catch (SecurityException e) {
		message[1] = "  java.vendor: INACCESSIBLE";
	    }
	    try {
		message[2] = "  java.vendor.url: " + System.getProperty("java.vendor.url");
	    } catch (SecurityException e) {
		message[2] = "  java.vendor.url: INACCESSIBLE";
	    }
	    try {
		message[3] = "  java.home: " + System.getProperty("java.home");
	    } catch (SecurityException e) {
		message[3] = "  java.home: INACCESSIBLE";
	    }
	    try {
		message[4] = "  java.class.version: " + System.getProperty("java.class.version");
	    } catch (SecurityException e) {
		message[4] = "  java.class.version: INACCESSIBLE";
	    }
	    try {
		message[5] = "  os.name: " + System.getProperty("os.name");
	    } catch (SecurityException e) {
		message[5] = "  os.name: INACCESSIBLE";
	    }
	    try {
		message[6] = "  os.arch: " + System.getProperty("os.arch");
	    } catch (SecurityException e) {
		message[6] = "  os.arch: INACCESSIBLE";
	    }
	    try {
		message[7] = "  os.version: " + System.getProperty("os.version");
	    } catch (SecurityException e) {
		message[7] = "  os.version: INACCESSIBLE";
	    }
	    try {
		message[8] = "  file.separator: " + System.getProperty("file.separator");
	    } catch (SecurityException e) {
		message[8] = "  file.separator: INACCESSIBLE";
	    }
	    try {
		message[9] = "  path.separator: " + System.getProperty("path.separator");
	    } catch (SecurityException e) {
		message[9] = "  path.separator: INACCESSIBLE";
	    }
	    try {
		message[10] = "  line.separator: " + System.getProperty("line.separator");
	    } catch (SecurityException e) {
		message[10] = "  line.separator: INACCESSIBLE";
	    }
	    try {
		message[11] = "  user.name: " + System.getProperty("user.name");
	    } catch (SecurityException e) {
		message[11] = "  user.name: INACCESSIBLE";
	    }
	    try {
		message[12] = "  user.home: " + System.getProperty("user.home");
	    } catch (SecurityException e) {
		message[12] = "  user.home: INACCESSIBLE";
	    }
	    try {
		message[13] = "  user.dir: " + System.getProperty("user.dir");
	    } catch (SecurityException e) {
		message[13] = "  user.dir: INACCESSIBLE";
	    }
	    for (int i = 0; i < 14; i++) {
		just[i] = Label.LEFT;
	    }

	    OkayDialog javaInfoDialog = new OkayDialog(this, 
		"About Java VM", 14, message, just, false);
	    javaInfoDialog.setResizable(false);
	    r = getBounds();
	    d = javaInfoDialog.getSize();
	    javaInfoDialog.setLocation((r.width - d.width) / 2 + r.x, 
		(r.height - d.height) / 2 + r.y);
	    javaInfoDialog.pack();
	    javaInfoDialog.show();
	} else if (whichOne == aboutMenuItem) {
	    // Bring up copyright dialog
	    Dimension d;
	    Rectangle r;
	    char copyright = 169;
	    String message[] = new String[7];
	    int just[] = new int[7];

	    message[0] = "SpiralGlyphics: Cyclic Glyph Viewer (v1.4)";
	    message[1] = "";
	    message[2] = "Copyright " + copyright + " 1998 by Benjamin Lipchak,";
	    message[3] = "Compaq Computer Corporation,";
	    message[4] = "Worcester Polytechnic Institute.";
	    message[5] = "All rights reserved.";
	    message[6] = "";
	    just[0] = just[1] = just[2] = just[3] = 
		just[4] = just[5] = just[6] = Label.CENTER;

	    OkayDialog aboutDialog = new OkayDialog(this, 
		"About SpiralGlyphics", 7, message, just, false);
	    aboutDialog.setResizable(false);
	    r = getBounds();
	    d = aboutDialog.getSize();
	    aboutDialog.setLocation((r.width - d.width) / 2 + r.x, 
		(r.height - d.height) / 2 + r.y);
	    aboutDialog.pack();
	    aboutDialog.show();
	} else {
	    // Bring up Not Implemented dialog
	    Dimension d;
	    Rectangle r;
	    String message[] = new String[4];
	    int just[] = new int[4];

	    message[0] = "";
	    message[1] = "This function has not";
	    message[2] = "yet been implemented.";
	    message[3] = "";
	    just[0] = just[1] = just[2] = just[3] = Label.CENTER;

	    OkayDialog nyiDialog = new OkayDialog(this, 
		"Not Yet Implemented", 4, message, just, true);
	    nyiDialog.setResizable(false);
	    r = getBounds();
	    d = nyiDialog.getSize();
	    nyiDialog.setLocation((r.width - d.width) / 2 + r.x, 
		(r.height - d.height) / 2 + r.y);
	    nyiDialog.pack();
	    nyiDialog.show();

	    this.toFront();
	}
    }

    public synchronized void itemStateChanged(ItemEvent ev) {
	ItemSelectable whichOne = ev.getItemSelectable();

	for (int i = 0; i < global.numLayoutPlugins; i++) {
	    int numOptions = global.layoutPlugins[i].getNumOptions();
	    if (whichOne == layoutCheckboxMenuItems[i]) {
		if (global.current.layoutType != i) {
		    layoutCheckboxMenuItems[i].setState(true);
		    // Enable this layout type's options
		    for (int j = 0; j < numOptions; j++)
			layoutSpecificCheckboxMenuItems[i][j].setEnabled(true);
		    for (int j = 0; j < global.numLayoutPlugins; j++) {
			if (i != j) {
			    layoutCheckboxMenuItems[j].setState(false);
			    // Disable options for other layout types
			    numOptions = global.layoutPlugins[j].getNumOptions();
			    for (int k = 0; k < numOptions; k++)
				layoutSpecificCheckboxMenuItems[j][k].setEnabled(false);
			}
		    }
		    global.sideControls.layoutTypeGroup.setSelectedCheckbox(
			global.sideControls.layoutTypeBoxes[i]);

		    global.current.layoutType = i;
		    global.layoutTypeDirty = true;
		    global.canvas.repaint();
		}
		break;
	    }

	    // Check for this layout type's options
	    for (int j = 0; j < numOptions; j++) {
		if (whichOne == layoutSpecificCheckboxMenuItems[i][j]) {
		    if (layoutSpecificCheckboxMenuItems[i][j].getState()) {
			if (!global.layoutPlugins[i].getOptionValue(j)) {
			    global.layoutPlugins[i].setOptionValue(j, true);
			    global.canvas.repaint();
			}
		    } else {
			if (global.layoutPlugins[i].getOptionValue(j)) {
			    global.layoutPlugins[i].setOptionValue(j, false);
			    global.canvas.repaint();
			}
		    }
		    break;
		}
	    }
	}

	for (int i = 0; i < global.numGlyphPlugins; i++) {
	    int numOptions = global.glyphPlugins[i].getNumOptions();
	    if (whichOne == glyphCheckboxMenuItems[i]) {
		if (global.current.glyphType != i) {
		    glyphCheckboxMenuItems[i].setState(true);
		    // Enable this glyph type's options
		    for (int j = 0; j < numOptions; j++)
			glyphSpecificCheckboxMenuItems[i][j].setEnabled(true);
		    for (int j = 0; j < global.numGlyphPlugins; j++) {
			if (i != j) {
			    glyphCheckboxMenuItems[j].setState(false);
			    // Disable options for other glyph types
			    numOptions = global.glyphPlugins[j].getNumOptions();
			    for (int k = 0; k < numOptions; k++)
				glyphSpecificCheckboxMenuItems[j][k].setEnabled(false);
			}
		    }
		    global.sideControls.glyphTypeGroup.setSelectedCheckbox(
			global.sideControls.glyphTypeBoxes[i]);

		    global.current.glyphType = i;
		    global.sideControls.glyphLegendCanvas.repaint();
		    global.canvas.repaint();
		}
		break;
	    }

	    // Check for this glyph type's options
	    for (int j = 0; j < numOptions; j++) {
		if (whichOne == glyphSpecificCheckboxMenuItems[i][j]) {
		    if (glyphSpecificCheckboxMenuItems[i][j].getState()) {
			if (!global.glyphPlugins[i].getOptionValue(j)) {
			    global.glyphPlugins[i].setOptionValue(j, true);
			    global.canvas.repaint();
			    global.sideControls.glyphLegendCanvas.repaint();
			}
		    } else {
			if (global.glyphPlugins[i].getOptionValue(j)) {
			    global.glyphPlugins[i].setOptionValue(j, false);
			    global.canvas.repaint();
			    global.sideControls.glyphLegendCanvas.repaint();
			}
		    }
		    break;
		}
	    }
	}

	if (whichOne == autoNormCheckboxMenuItem)
	{
	   if (autoNormCheckboxMenuItem.getState()) {
		if (!global.current.autoNormalize) {
		    global.current.autoNormalize = true;
		    global.canvas.repaint();
		}
	   } else {
		if (global.current.autoNormalize) {
		    global.current.autoNormalize = false;
		    global.canvas.repaint();
		}
	   }
	} else if (whichOne == labelsCheckboxMenuItem)
	{
	   if (labelsCheckboxMenuItem.getState()) {
		if (!global.current.showLabels) {
		    global.current.showLabels = true;
		    global.canvas.repaint();
		}
	   } else {
		if (global.current.showLabels) {
		    global.current.showLabels = false;
		    global.canvas.repaint();
		}
	   }
	} else if (whichOne == guidelinesCheckboxMenuItem)
	{
	   if (guidelinesCheckboxMenuItem.getState()) {
		if (!global.current.showGuidelines) {
		    global.current.showGuidelines = true;
		    global.canvas.repaint();
		}
	   } else {
		if (global.current.showGuidelines) {
		    global.current.showGuidelines = false;
		    global.canvas.repaint();
		}
	   }
	} else if (whichOne == fillGlyphsCheckboxMenuItem)
	{
	   if (fillGlyphsCheckboxMenuItem.getState()) {
		if (!global.current.fillGlyphs) {
		    global.current.fillGlyphs = true;
		    global.sideControls.glyphLegendCanvas.repaint();
		    global.canvas.repaint();
		}
	   } else {
		if (global.current.fillGlyphs) {
		    global.current.fillGlyphs = false;
		    global.sideControls.glyphLegendCanvas.repaint();
		    global.canvas.repaint();
		}
	   }
	} else if (whichOne == colorPerDimCheckboxMenuItem) 
	{
	   colorPerDimCheckboxMenuItem.setState(true);
	   if (global.current.colorPerGlyph) {
		colorPerGlyphCheckboxMenuItem.setState(false);
		global.current.colorPerGlyph = false;
		global.sideControls.glyphLegendCanvas.repaint();
		global.canvas.repaint();
	   }
	} else if (whichOne == colorPerGlyphCheckboxMenuItem) 
	{
	   colorPerGlyphCheckboxMenuItem.setState(true);
	   if (!global.current.colorPerGlyph) {
		colorPerDimCheckboxMenuItem.setState(false);
		global.current.colorPerGlyph = true;
		global.sideControls.glyphLegendCanvas.repaint();
		global.canvas.repaint();
	   }
	}
    }
}
