//
// ViewerGlyphLegendCanvas.java
//
// Here lies the code that is responsible
// for drawing the legend which represents
// the current glyph attributes...
//

package spg;

import java.awt.*;

public class ViewerGlyphLegendCanvas extends Canvas {
    static int LEGEND_FONT_SIZE = 14;

    ViewerState global;
    int sideLength, xOrigin, yOrigin;
    int halfSideLength;

    Font legendFont;
    FontMetrics legendFM;

    Dimension offDimension;
    Graphics offGraphics;
    Image offImage;

    public ViewerGlyphLegendCanvas(ViewerState gInfo) {
	 global = gInfo;
    }

    public void paint(Graphics g) {
	update(g);
    }

    public void update(Graphics g) {
	Dimension d = getSize();

	if ((offGraphics == null) || (d.width != offDimension.width) ||
	   (d.height != offDimension.height))
	{
	   offDimension = d;
	   offImage = createImage(d.width, d.height);
	   offGraphics = offImage.getGraphics();

	   legendFont = new Font("SansSerif", Font.BOLD, LEGEND_FONT_SIZE);
	   legendFM = offGraphics.getFontMetrics(legendFont);
	}

	if (d.height < d.width)
	    sideLength = d.height;
	else
	    sideLength = d.width;

	halfSideLength = sideLength / 2;
	xOrigin = halfSideLength;
	yOrigin = halfSideLength;

	// Set clipRect
	offGraphics.setClip(0, 0, sideLength, sideLength);

	// Clear background
	offGraphics.setColor(Color.white);
	offGraphics.fillRect(0, 0, sideLength, sideLength);

	offGraphics.setColor(Color.black);
	offGraphics.drawRect(0, 0, sideLength - 1, sideLength - 1);
 
	// If no data set loaded, indicate this!
	if (global.fileName == null) {
	    int labelX, labelY;
	    int labelWidth, labelHeight;

	    offGraphics.setColor(Color.black);
	    offGraphics.setFont(legendFont);

	    labelHeight = legendFM.getAscent();
	    labelWidth = legendFM.stringWidth("Open a");
	    labelX = xOrigin - (labelWidth / 2);
	    labelY = yOrigin - (labelHeight / 2);
	    offGraphics.drawString("Open a", labelX, labelY);

	    labelWidth = legendFM.stringWidth("data file.");
	    labelX = xOrigin - (labelWidth / 2);
	    labelY = yOrigin + (labelHeight * 3) / 2;
	    offGraphics.drawString("data file.", labelX, labelY);

	    // Swap buffers
	    g.drawImage(offImage, 0, 0, this);

	    return;
	}

	// If no dimensions enabled, get the hell out of Dodge
	if (global.current.enabledDim == 0) {
	    int labelX, labelY;
	    int labelWidth, labelHeight;

	    offGraphics.setColor(Color.black);
	    offGraphics.setFont(legendFont);

	    labelHeight = legendFM.getAscent();
	    labelWidth = legendFM.stringWidth("No dim's");
	    labelX = xOrigin - (labelWidth / 2);
	    labelY = yOrigin - (labelHeight / 2);
	    offGraphics.drawString("No dim's", labelX, labelY);

	    labelWidth = legendFM.stringWidth("enabled!");
	    labelX = xOrigin - (labelWidth / 2);
	    labelY = yOrigin + (labelHeight * 3) / 2;
	    offGraphics.drawString("enabled!", labelX, labelY);

	    // Swap buffers
	    g.drawImage(offImage, 0, 0, this);

	    return;
	}

	// If our canvas isn't big enough for legend, say so.
	if (sideLength < 60) {
	    int labelX, labelY;
	    int labelWidth, labelHeight;

	    offGraphics.setColor(Color.darkGray);
	    offGraphics.setFont(legendFont);

	    labelHeight = legendFM.getAscent();
	    labelWidth = legendFM.stringWidth("No room for");
	    labelX = xOrigin - (labelWidth / 2);
	    labelY = yOrigin - (labelHeight / 2);
	    offGraphics.drawString("No room for", labelX, labelY);

	    labelWidth = legendFM.stringWidth("glyph legend!");
	    labelX = xOrigin - (labelWidth / 2);
	    labelY = yOrigin + (labelHeight * 3) / 2;
	    offGraphics.drawString("glyph legend!", labelX, labelY);

	    // Swap buffers
	    g.drawImage(offImage, 0, 0, this);

	    return;
	}

	// Draw the legend
	global.glyphPlugins[global.current.glyphType].setLegendFontInfo(legendFont, legendFM);
	global.glyphPlugins[global.current.glyphType].drawLegend(offGraphics, sideLength, global);

	// Swap buffers
	g.drawImage(offImage, 0, 0, this);
    }
}
