//
// ViewerSideControls.java
//
// These are the GUI controls that appear
// to the left of the data window...
//

package spg;

import java.awt.*;
import java.awt.event.*;

public class ViewerSideControls extends Panel
                  implements ItemListener, AdjustmentListener, 
								ActionListener {
    ViewerState global;

    Label dummy0, dummy1, dummy2, dummy3;

    Panel checkboxPanel, glyphLegendPanel, dimLabelsPanel;
    ScrollPane dimLabelsScrollPane;
    GridBagLayout dimLabelsGridBag;
    GridBagConstraints dimLabelsC;

    ViewerGlyphLegendCanvas glyphLegendCanvas;
    Checkbox dimensionLabelBox[];
    Button dimensionUpButton[];
    Button dimensionDownButton[];

    Label layoutTypeLabel;
    Checkbox[] layoutTypeBoxes;
    CheckboxGroup layoutTypeGroup;

    Label glyphTypeLabel;
    Checkbox[] glyphTypeBoxes;
    CheckboxGroup glyphTypeGroup;

    Scrollbar visibleCyclesScrollbar;
    Scrollbar cycleLengthScrollbar;
    Scrollbar glyphScaleScrollbar;

    TextField vcsVal, clsVal, gssVal;
    Label vcsMin, vcsMax, vcsLabel;
    Label clsMin, clsMax, clsLabel;
    Label gssMin, gssMax, gssLabel;

    public ViewerSideControls(ViewerState gInfo) {
	global = gInfo;

	GridBagLayout gridBag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gridBag);

	c.gridx = 0;
	c.gridwidth = 3;
	c.insets = new Insets(0,10,0,10);
	c.weighty = 0.5;
	c.gridy = 1;
	dummy0 = new Label("");
	gridBag.setConstraints(dummy0, c);
	c.gridy = 3;
	dummy1 = new Label("");
	gridBag.setConstraints(dummy1, c);

	c.weighty = 0.25;
	c.gridy = 7;
	dummy2 = new Label("");
	gridBag.setConstraints(dummy2, c);
	c.gridy = 11;
	dummy3 = new Label("");
	gridBag.setConstraints(dummy3, c);

	glyphLegendCanvas = new ViewerGlyphLegendCanvas(global);
	GridBagLayout legendGridBag = new GridBagLayout();
	GridBagConstraints legendC = new GridBagConstraints();
	glyphLegendPanel = new Panel();
	dimLabelsScrollPane = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
	dimLabelsGridBag = new GridBagLayout();
	dimLabelsC = new GridBagConstraints();
	dimLabelsPanel = new Panel(dimLabelsGridBag);
	dimLabelsScrollPane.add(dimLabelsPanel);
	glyphLegendPanel.setLayout(legendGridBag);
	legendC.gridheight = GridBagConstraints.REMAINDER;
	legendC.fill = GridBagConstraints.BOTH;
	legendC.weightx = 1.0;
	legendC.weighty = 1.0;
	legendGridBag.setConstraints(glyphLegendCanvas, legendC);
	glyphLegendPanel.add(glyphLegendCanvas);
	legendC.insets = new Insets(0,10,0,0);
	legendC.weightx = 0.4;
	legendGridBag.setConstraints(dimLabelsScrollPane, legendC);
	glyphLegendPanel.add(dimLabelsScrollPane);
	c.insets = new Insets(10,10,0,10);
	c.gridy = 0;
	c.weighty = 0.5;
	c.fill = GridBagConstraints.BOTH;
	gridBag.setConstraints(glyphLegendPanel, c);

	layoutTypeLabel = new Label("Layout Type:");
	layoutTypeGroup = new CheckboxGroup();
	layoutTypeBoxes = new Checkbox[global.numLayoutPlugins];
	for (int i = 0; i < global.numLayoutPlugins; i++) {
	    layoutTypeBoxes[i] = new Checkbox(global.layoutPlugins[i].getName(),
		(global.current.layoutType == i), layoutTypeGroup);
	    layoutTypeBoxes[i].setEnabled(false);
	    layoutTypeBoxes[i].addItemListener(this);
	}

	glyphTypeLabel = new Label("Glyph Type:");
	glyphTypeGroup = new CheckboxGroup();
	glyphTypeBoxes = new Checkbox[global.numGlyphPlugins];
	for (int i = 0; i < global.numGlyphPlugins; i++) {
	    glyphTypeBoxes[i] = new Checkbox(global.glyphPlugins[i].getName(),
		(global.current.glyphType == i), glyphTypeGroup);
	    glyphTypeBoxes[i].setEnabled(false);
	    glyphTypeBoxes[i].addItemListener(this);
	}

	checkboxPanel = new Panel(new GridLayout(0,2,10,0));
	c.insets = new Insets(0,10,0,10);
	c.fill = GridBagConstraints.NONE;
	c.gridy = 2;
	c.weighty = 0.0;
	gridBag.setConstraints(checkboxPanel, c);
	checkboxPanel.add(layoutTypeLabel);
	checkboxPanel.add(glyphTypeLabel);
	
	int numRows = (global.numGlyphPlugins > global.numLayoutPlugins) ? 
	    global.numGlyphPlugins : global.numLayoutPlugins;
	for (int i = 0; i < numRows; i++) {
	    if (i < global.numLayoutPlugins) {
		checkboxPanel.add(layoutTypeBoxes[i]);
	    } else {
		checkboxPanel.add(new Label(""));
	    }
	    if (i < global.numGlyphPlugins) {
		checkboxPanel.add(glyphTypeBoxes[i]);
	    } else {
		checkboxPanel.add(new Label(""));
	    }
	}

	int vcsShow = (int)Math.ceil((double)global.derived.numCycles / 5.0);
	visibleCyclesScrollbar = new Scrollbar(Scrollbar.HORIZONTAL,
	  global.current.visibleCycles, vcsShow, 1, 
	  global.derived.numCycles + vcsShow);
	visibleCyclesScrollbar.setBlockIncrement(vcsShow); 
	visibleCyclesScrollbar.setEnabled(false);
	int clsMaxVal = 
	   (int)Math.rint(global.constant.tSpan * ViewerState.TSCROLLFACTOR);
	int clsShow = (int)Math.ceil((double)clsMaxVal / 5.0);
	cycleLengthScrollbar = new Scrollbar(Scrollbar.HORIZONTAL,
	  (int)Math.rint(global.current.cycleLength * ViewerState.TSCROLLFACTOR), 
	  clsShow, 
	  (int)Math.rint(global.constant.tMinimumUnit * ViewerState.TSCROLLFACTOR), 
	  clsMaxVal + clsShow); 
	cycleLengthScrollbar.setBlockIncrement((int)Math.rint(ViewerState.TSCROLLFACTOR));
	cycleLengthScrollbar.setEnabled(false);
	glyphScaleScrollbar = new Scrollbar(Scrollbar.HORIZONTAL,
	  (int)Math.rint(global.current.glyphScale * 100.0), 40, 0, 240);
	glyphScaleScrollbar.setBlockIncrement(40);
	glyphScaleScrollbar.setEnabled(false);
	visibleCyclesScrollbar.addAdjustmentListener(this);
	cycleLengthScrollbar.addAdjustmentListener(this);
	glyphScaleScrollbar.addAdjustmentListener(this);

	c.anchor = GridBagConstraints.CENTER;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.weightx = 0.0;
	c.gridwidth = 3;
	c.gridx = 0;
	c.gridy = 5;
	gridBag.setConstraints(visibleCyclesScrollbar, c);
	c.gridy = 9;
	gridBag.setConstraints(cycleLengthScrollbar, c);
	c.gridy = 13;
	gridBag.setConstraints(glyphScaleScrollbar, c);

	vcsMin = new Label("" + 1);
	vcsVal = new TextField("" + global.current.visibleCycles, 5);
	vcsVal.setEnabled(false);
	vcsVal.addActionListener(this);
	vcsMax = new Label("" + global.derived.numCycles, Label.RIGHT);
	vcsLabel = new Label("Number of Visible Cycles", Label.CENTER);
	clsMin = new Label("" + global.constant.tMinimumUnit);
	clsVal = new TextField("" + global.current.cycleLength, 5);
	clsVal.setEnabled(false);
	clsVal.addActionListener(this);
	clsMax = new Label("" + global.constant.tSpan, Label.RIGHT);
	clsLabel = new Label("Cycle Length", Label.CENTER);
	gssMin = new Label("" + 0.0);
	gssVal = new TextField("" + global.current.glyphScale, 5);
	gssVal.setEnabled(false);
	gssVal.setBackground(getBackground());
	gssVal.addActionListener(this);
	gssMax = new Label("" + 2.0, Label.RIGHT);
	gssLabel = new Label("Glyph Scale Factor", Label.CENTER);

	c.fill = GridBagConstraints.NONE;
	c.weightx = 1.0;
	c.ipadx = 5;
	c.gridwidth = 1;
	c.gridy = 4;
	gridBag.setConstraints(vcsMin, c);
	c.gridx = 1;
	gridBag.setConstraints(vcsVal, c);
	c.gridx = 2;
	gridBag.setConstraints(vcsMax, c);
	c.gridx = 0;
	c.gridy = 6;
	c.gridwidth = 3;
	gridBag.setConstraints(vcsLabel, c);
	c.gridwidth = 1;
	c.gridy = 8;
	gridBag.setConstraints(clsMin, c);
	c.gridx = 1;
	gridBag.setConstraints(clsVal, c);
	c.gridx = 2;
	gridBag.setConstraints(clsMax, c);
	c.gridx = 0;
	c.gridy = 10;
	c.gridwidth = 3;
	gridBag.setConstraints(clsLabel, c);
	c.gridwidth = 1;
	c.gridy = 12;
	gridBag.setConstraints(gssMin, c);
	c.gridx = 1;
	gridBag.setConstraints(gssVal, c);
	c.gridx = 2;
	gridBag.setConstraints(gssMax, c);
	c.gridx = 0;
	c.gridy = 14;
	c.gridwidth = 3;
	c.insets = new Insets(0, 10, 10, 10);
	gridBag.setConstraints(gssLabel, c);

	// add legend
	add(glyphLegendPanel);
	add(dummy0);
	add(checkboxPanel);
	add(dummy1);
	add(vcsMin); add(vcsVal); add(vcsMax);
	add(visibleCyclesScrollbar);
	add(vcsLabel);
	add(dummy2);
	add(clsMin); add(clsVal); add(clsMax);
	add(cycleLengthScrollbar);
	add(clsLabel);
	add(dummy3);
	add(gssMin); add(gssVal); add(gssMax);
	add(glyphScaleScrollbar);
	add(gssLabel);

	validate();
    }

    public synchronized void actionPerformed(ActionEvent ev) {
	Object whichOne = ev.getSource();
	if (whichOne == vcsVal) {
	    int newVisCycles;

	    try {
		newVisCycles = Integer.valueOf(vcsVal.getText()).intValue();
	    } catch (NumberFormatException e) {
		newVisCycles = global.current.visibleCycles;
	    }

	   if (newVisCycles > global.derived.numCycles)
		newVisCycles = global.derived.numCycles;
	   else if (newVisCycles < 1)
		newVisCycles = 1;

	   global.current.visibleCycles = newVisCycles;
	   vcsVal.setText("" + global.current.visibleCycles);
	   visibleCyclesScrollbar.setValue(global.current.visibleCycles);

	   global.derived.tInterval = global.current.cycleLength * 
		(double)global.current.visibleCycles;
	   if (global.derived.tInterval > global.constant.tSpan)
		global.derived.tInterval = global.constant.tSpan;

	   double tTempMax = global.constant.tMax - global.derived.tInterval + 
		global.constant.tMinimumUnit;
	   int tMaxVal = (int)Math.rint(tTempMax * ViewerState.TSCROLLFACTOR);
	   if (global.current.tVal > tTempMax)
	      global.current.tVal = ViewerState.massageDouble(tTempMax, 
		ViewerState.TSCROLLFACTOR);
	   global.derived.tVal2 = ViewerState.massageDouble(global.current.tVal +
		global.derived.tInterval, ViewerState.TSCROLLFACTOR);
	   global.bottomControls.tValLabel.setText("[" + global.current.tVal + ", " + 
	      global.derived.tVal2 + "]"); 

	   int tShow = (int)Math.rint(global.derived.tInterval * 
		ViewerState.TSCROLLFACTOR);
	   global.bottomControls.tScrollbar.setValues(
	      (int)Math.rint(global.current.tVal * ViewerState.TSCROLLFACTOR), 
	      tShow, 
	      (int)Math.rint(global.constant.tMin * ViewerState.TSCROLLFACTOR), 
	      tMaxVal + tShow); 
	   global.bottomControls.tScrollbar.setBlockIncrement(
	      (int)Math.ceil(global.current.cycleLength * ViewerState.TSCROLLFACTOR));

	   global.canvas.repaint();
	} else if (whichOne == clsVal) {
	    double newLength;

	    try {
		newLength = Double.valueOf(clsVal.getText()).doubleValue();
	    } catch (NumberFormatException e) {
		newLength = global.current.cycleLength;
	    }

	   if (newLength > global.constant.tSpan)
		newLength = global.constant.tSpan;
	   else if (newLength < 0.0)
		newLength = 0.0;

	   global.current.cycleLength = newLength;
	   clsVal.setText("" + global.current.cycleLength);
	   cycleLengthScrollbar.setValue(
		(int)Math.rint(global.current.cycleLength * ViewerState.TSCROLLFACTOR));

	   global.derived.tInterval = global.current.cycleLength * 
		(double)global.current.visibleCycles;
	   if (global.derived.tInterval > global.constant.tSpan)
		global.derived.tInterval = global.constant.tSpan;

	   global.derived.numCycles = 
	      (int)Math.ceil(global.constant.tSpan / global.current.cycleLength);
	   vcsMax.setText("" + global.derived.numCycles);
	   if (global.current.visibleCycles > global.derived.numCycles) {
	      global.current.visibleCycles = global.derived.numCycles;
		vcsVal.setText("" + global.current.visibleCycles);
	   }
	   int vcsShow = (int)Math.ceil((double)global.derived.numCycles / 5.0);
	   visibleCyclesScrollbar.setValues(global.current.visibleCycles, 
	      vcsShow, 1, global.derived.numCycles + vcsShow);
	   visibleCyclesScrollbar.setBlockIncrement(vcsShow); 

	   double tTempMax = global.constant.tMax - global.derived.tInterval + 
		global.constant.tMinimumUnit;
	   int tMaxVal = (int)Math.rint(tTempMax * ViewerState.TSCROLLFACTOR);
	   if (global.current.tVal > tTempMax)
	      global.current.tVal = ViewerState.massageDouble(tTempMax, 
		ViewerState.TSCROLLFACTOR);
	   global.derived.tVal2 = ViewerState.massageDouble(global.current.tVal + 
		global.derived.tInterval, ViewerState.TSCROLLFACTOR);
	   global.bottomControls.tValLabel.setText("[" + global.current.tVal + ", " + 
	      global.derived.tVal2 + "]"); 

	   int tShow = (int)Math.rint(global.derived.tInterval * 
		ViewerState.TSCROLLFACTOR);
	   global.bottomControls.tScrollbar.setValues(
	      (int)Math.rint(global.current.tVal * ViewerState.TSCROLLFACTOR), 
	      tShow, 
	      (int)Math.rint(global.constant.tMin * ViewerState.TSCROLLFACTOR), 
	      tMaxVal + tShow); 
	   global.bottomControls.tScrollbar.setBlockIncrement(
	      (int)Math.rint(global.current.cycleLength * ViewerState.TSCROLLFACTOR));

	   global.canvas.repaint();
	} else if (whichOne == gssVal) {
	    double newScale;

	    try {
		newScale = Double.valueOf(gssVal.getText()).doubleValue();
	    } catch (NumberFormatException e) {
		newScale = global.current.glyphScale;
	    }

	   if (newScale > 2.0)
		newScale = 2.0;
	   else if (newScale < 0.0)
		newScale = 0.0;

	   global.current.glyphScale = newScale;
	   gssVal.setText("" + global.current.glyphScale);

	   glyphScaleScrollbar.setValue(
		(int)Math.rint(global.current.glyphScale * 100.0));

	   global.canvas.repaint();
	} else {
	    for (int i = 0; i < global.constant.dataDim; i++) {
		if (whichOne == dimensionUpButton[i]) {
		    int swapNext = i + 1;
		    while (!dimensionLabelBox[swapNext].getState())
			swapNext++;

		    int swapValue = global.current.listMap[swapNext];
		    global.current.listMap[swapNext] = global.current.listMap[i];
		    global.current.listMap[i] = swapValue;

		    global.current.enabledDim = 0;
		    int lastEnabled = 0;
		    for (int j = 0; j < global.constant.dataDim; j++) {
			if (dimensionLabelBox[j].getState()) {
			    global.current.enabledDim++;
			    dimensionLabelBox[j].setLabel("" + 
				global.current.enabledDim + ". " + 
				global.constant.dimLabel[global.current.listMap[j]]);
			    global.current.dimMap[global.current.enabledDim] = 
				1 + global.current.listMap[j];
			    if (global.current.enabledDim > 1)
				dimensionDownButton[j].setEnabled(true);
			    else
				dimensionDownButton[j].setEnabled(false);
			    dimensionUpButton[j].setEnabled(true);
			    lastEnabled = j;
			} else {
			    dimensionLabelBox[j].setLabel("    " + 
				global.constant.dimLabel[global.current.listMap[j]]);
			    dimensionUpButton[j].setEnabled(false);
			    dimensionDownButton[j].setEnabled(false);
			}
		    }
		    if (global.current.enabledDim > 0) {
			dimensionUpButton[lastEnabled].setEnabled(false);
		    }

		    global.sideControls.glyphLegendCanvas.repaint();
		    global.canvas.repaint();

		    break;
		} else if (whichOne == dimensionDownButton[i]) {
		    int swapPrev = i - 1;
		    while (!dimensionLabelBox[swapPrev].getState())
			swapPrev--;

		    int swapValue = global.current.listMap[swapPrev];
		    global.current.listMap[swapPrev] = global.current.listMap[i];
		    global.current.listMap[i] = swapValue;

		    global.current.enabledDim = 0;
		    int lastEnabled = 0;
		    for (int j = 0; j < global.constant.dataDim; j++) {
			if (dimensionLabelBox[j].getState()) {
			    global.current.enabledDim++;
			    dimensionLabelBox[j].setLabel("" + 
				global.current.enabledDim + ". " + 
				global.constant.dimLabel[global.current.listMap[j]]);
			    global.current.dimMap[global.current.enabledDim] = 
				1 + global.current.listMap[j];
			    if (global.current.enabledDim > 1)
				dimensionDownButton[j].setEnabled(true);
			    else
				dimensionDownButton[j].setEnabled(false);
			    dimensionUpButton[j].setEnabled(true);
			    lastEnabled = j;
			} else {
			    dimensionLabelBox[j].setLabel("    " + 
				global.constant.dimLabel[global.current.listMap[j]]);
			    dimensionUpButton[j].setEnabled(false);
			    dimensionDownButton[j].setEnabled(false);
			}
		    }
		    if (global.current.enabledDim > 0) {
			dimensionUpButton[lastEnabled].setEnabled(false);
		    }

		    global.sideControls.glyphLegendCanvas.repaint();
		    global.canvas.repaint();

		    break;
		}
	    }
	}
    }

    public synchronized void itemStateChanged(ItemEvent ev) {
	ItemSelectable whichOne = ev.getItemSelectable();
	for (int i = 0; i < global.numLayoutPlugins; i++) {
	    if (whichOne == layoutTypeBoxes[i] && layoutTypeBoxes[i].getState()) {
		if (global.current.layoutType != i) {
		    for (int j = 0; j < global.numLayoutPlugins; j++) {
			int numOptions = global.layoutPlugins[j].getNumOptions();
			if (i == j) {
			    global.frame.layoutCheckboxMenuItems[j].setState(true);
			    // Enable this layout type's options
			    for (int k = 0; k < numOptions; k++)
				global.frame.layoutSpecificCheckboxMenuItems[j][k].setEnabled(true);
			} else {
			    global.frame.layoutCheckboxMenuItems[j].setState(false);
			    // Disable options for other layout types
			    for (int k = 0; k < numOptions; k++)
				global.frame.layoutSpecificCheckboxMenuItems[j][k].setEnabled(false);
			}
		    }
		    global.current.layoutType = i;
		    global.layoutTypeDirty = true;
		    global.canvas.repaint();
		}
		break;
	    }
	}
	for (int i = 0; i < global.numGlyphPlugins; i++) {
	    if (whichOne == glyphTypeBoxes[i] && glyphTypeBoxes[i].getState()) {
		if (global.current.glyphType != i) {
		    for (int j = 0; j < global.numGlyphPlugins; j++) {
			int numOptions = global.glyphPlugins[j].getNumOptions();
			if (i == j) {
			    global.frame.glyphCheckboxMenuItems[j].setState(true);
			    // Enable this glyph type's options
			    for (int k = 0; k < numOptions; k++)
				global.frame.glyphSpecificCheckboxMenuItems[j][k].setEnabled(true);
			} else {
			    global.frame.glyphCheckboxMenuItems[j].setState(false);
			    // Disable options for other glyph types
			    for (int k = 0; k < numOptions; k++)
				global.frame.glyphSpecificCheckboxMenuItems[j][k].setEnabled(false);
			}
		    }
		    global.current.glyphType = i;
		    global.sideControls.glyphLegendCanvas.repaint();
		    global.canvas.repaint();
		}
		break;
	    }
	}
 	for (int i = 0; i < global.constant.dataDim; i++) {
	    if (whichOne == dimensionLabelBox[i]) {
		global.current.enabledDim = 0;
		int lastEnabled = 0;
		for (int j = 0; j < global.constant.dataDim; j++) {
		    if (dimensionLabelBox[j].getState()) {
			global.current.enabledDim++;
			dimensionLabelBox[j].setLabel("" + 
			    global.current.enabledDim + ". " + 
			    global.constant.dimLabel[global.current.listMap[j]]);
			global.current.dimMap[global.current.enabledDim] = 
			    1 + global.current.listMap[j];
			if (global.current.enabledDim > 1)
			    dimensionDownButton[j].setEnabled(true);
			else
			    dimensionDownButton[j].setEnabled(false);
			dimensionUpButton[j].setEnabled(true);
			lastEnabled = j;
		    } else {
			dimensionLabelBox[j].setLabel("    " + 
			    global.constant.dimLabel[global.current.listMap[j]]);
			dimensionUpButton[j].setEnabled(false);
			dimensionDownButton[j].setEnabled(false);
		    }
		}
		if (global.current.enabledDim > 0) {
		    dimensionUpButton[lastEnabled].setEnabled(false);
		}

		global.sideControls.glyphLegendCanvas.repaint();
		global.canvas.repaint();

		break;
	    }
	}
    }

    public synchronized void adjustmentValueChanged(AdjustmentEvent ev) {
	Adjustable whichOne = ev.getAdjustable();
	if (whichOne == visibleCyclesScrollbar) {
	   global.current.visibleCycles = visibleCyclesScrollbar.getValue();
	   vcsVal.setText("" + global.current.visibleCycles);

	   global.derived.tInterval = global.current.cycleLength * 
		(double)global.current.visibleCycles;
	   if (global.derived.tInterval > global.constant.tSpan)
		global.derived.tInterval = global.constant.tSpan;

	   double tTempMax = global.constant.tMax - global.derived.tInterval + 
		global.constant.tMinimumUnit;
	   int tMaxVal = (int)Math.rint(tTempMax * ViewerState.TSCROLLFACTOR);
	   if (global.current.tVal > tTempMax)
	      global.current.tVal = ViewerState.massageDouble(tTempMax, 
		ViewerState.TSCROLLFACTOR);
	   global.derived.tVal2 = ViewerState.massageDouble(global.current.tVal + 
		global.derived.tInterval, ViewerState.TSCROLLFACTOR);
	   global.bottomControls.tValLabel.setText("[" + global.current.tVal + ", " + 
	      global.derived.tVal2 + "]"); 

	   int tShow = (int)Math.rint(global.derived.tInterval * 
		ViewerState.TSCROLLFACTOR);
	   global.bottomControls.tScrollbar.setValues(
	      (int)Math.rint(global.current.tVal * ViewerState.TSCROLLFACTOR), 
	      tShow, 
	      (int)Math.rint(global.constant.tMin * ViewerState.TSCROLLFACTOR), 
	      tMaxVal + tShow); 
	   global.bottomControls.tScrollbar.setBlockIncrement(
	      (int)Math.ceil(global.current.cycleLength * ViewerState.TSCROLLFACTOR));

	   global.canvas.repaint();	// THROTTLE HERE?
	} else if (whichOne == cycleLengthScrollbar) {
	   double newCycleLength = ((double)cycleLengthScrollbar.getValue() / 
		ViewerState.TSCROLLFACTOR);
	   clsVal.setText("" + newCycleLength);

	   global.derived.tInterval = newCycleLength * 
		(double)global.current.visibleCycles;
	   if (global.derived.tInterval > global.constant.tSpan)
		global.derived.tInterval = global.constant.tSpan;

	   global.derived.numCycles = 
	      (int)Math.ceil(global.constant.tSpan / newCycleLength);
	   vcsMax.setText("" + global.derived.numCycles);
	   if (global.current.visibleCycles > global.derived.numCycles)
	      global.current.visibleCycles = global.derived.numCycles;
	   vcsVal.setText("" + global.current.visibleCycles);
	   int vcsShow = (int)Math.ceil((double)global.derived.numCycles / 5.0);
	   visibleCyclesScrollbar.setValues(global.current.visibleCycles, 
	      vcsShow, 1, global.derived.numCycles + vcsShow);
	   visibleCyclesScrollbar.setBlockIncrement(vcsShow); 

	   double tTempMax = global.constant.tMax - global.derived.tInterval + 
		global.constant.tMinimumUnit;
	   int tMaxVal = (int)Math.rint(tTempMax * ViewerState.TSCROLLFACTOR);
	   if (global.current.tVal > tTempMax)
	      global.current.tVal = ViewerState.massageDouble(tTempMax, 
		ViewerState.TSCROLLFACTOR);
	   global.derived.tVal2 = ViewerState.massageDouble(global.current.tVal + 
		global.derived.tInterval, ViewerState.TSCROLLFACTOR);
	   global.bottomControls.tValLabel.setText("[" + global.current.tVal + ", " + 
	      global.derived.tVal2 + "]"); 

	   int tShow = (int)Math.rint(global.derived.tInterval * 
		ViewerState.TSCROLLFACTOR);
	   global.bottomControls.tScrollbar.setValues(
	      (int)Math.rint(global.current.tVal * ViewerState.TSCROLLFACTOR), 
	      tShow, 
	      (int)Math.rint(global.constant.tMin * ViewerState.TSCROLLFACTOR), 
	      tMaxVal + tShow); 
	   global.bottomControls.tScrollbar.setBlockIncrement(
	      (int)Math.rint(global.current.cycleLength * ViewerState.TSCROLLFACTOR));

	   global.current.cycleLength = newCycleLength;
	   global.canvas.repaint();	// THROTTLE HERE?
	} else if (whichOne == glyphScaleScrollbar) {
	   global.current.glyphScale = (double)glyphScaleScrollbar.getValue() / 100.0;
	   gssVal.setText("" + global.current.glyphScale);

	   global.canvas.repaint();	// THROTTLE HERE?
	} 
    }
}
	
