//
// LinearLayoutPlugin.java
//
// This file provides the code used to extend the DataLayout class to
// plot the data on a linear graph.
//

package spg.plugins;

import spg.*;
import java.awt.*;

public class LinearLayoutPlugin extends DataLayout {
    private int yOrigin, glyphX, prevGlyphX;
    private int plotOffset, plotScale;
    private int halfGlyphSizeInt;
    private double halfGlyphSize;
    private double cycleWidth, cycleOffset;
    private double tVal;
    private double dataPoint[], prevDataPoint[];

    public LinearLayoutPlugin() {
	super("Linear Layout");

	numOptions = 2;
	optionFlags = new boolean[numOptions];
	optionNames = new String[numOptions];
	optionFlags[0] = true;
	optionNames[0] = "Show Plot";
	optionFlags[1] = true;
	optionNames[1] = "Line Plot";

	plotOffset = plotScale = yOrigin = glyphX = prevGlyphX = halfGlyphSizeInt = 0;
	halfGlyphSize = cycleWidth = cycleOffset = tVal = 0.0;
	dataPoint = prevDataPoint = null;
    }

    public Rectangle getLayoutBounds(Dimension d) {
	bounds.width = d.width - 20;
	bounds.x = 10;
	if (d.width > d.height) {
	   bounds.height = d.height - 20;
	   bounds.y = 10;
	} else {
	   bounds.height = bounds.width;
	   bounds.y = (d.height - bounds.height) / 2;
	}

	return bounds;
    }

    public void drawLayout(Graphics g, ViewerState global) {
	currentGlyphType = global.getGlyphPlugin();

	if (optionFlags[0]) {
	    yOrigin = bounds.y + (7 * bounds.height / 8);
	    plotOffset = bounds.y + (bounds.height / 8);
	    plotScale = bounds.height * 5 / 8;
	} else
	    yOrigin = bounds.y + (bounds.height / 2);

	halfGlyphSize = (double)(bounds.width - 10) * 0.04 /
	    (double)global.getVisibleCycles();
	halfGlyphSize *= (global.getGlyphScale() / 
	    (greatestValue - leastValue));
	halfGlyphSizeInt = (int)Math.rint(halfGlyphSize);

	cycleWidth = ((double)bounds.width - (halfGlyphSize * 2.0)) / 
	    (double)global.getVisibleCycles();
	cycleOffset = Math.IEEEremainder(global.getTVal() - global.getTMin(), 
	    global.getCycleLength());
	if (cycleOffset < 0.0)
	    cycleOffset += global.getCycleLength();
	cycleOffset /= global.getCycleLength();

	// Highlight brushed offsets
	if (global.getBrushOffsetX() >= 0) {
	    if (global.getBrushOffset() == -1.0) {
		global.setBrushOffset((int)Math.rint(global.getBrushOffsetX() -
		    bounds.x - halfGlyphSizeInt));
		global.setBrushOffset(Math.IEEEremainder(global.getBrushOffset(), 
		    cycleWidth));
		if (global.getBrushOffset() < 0.0)
		    global.setBrushOffset(global.getBrushOffset() + cycleWidth);
		global.setBrushOffset(global.getBrushOffset() / cycleWidth);
		global.setBrushOffset(global.getBrushOffset() + cycleOffset);
		if (global.getBrushOffset() > 1.0)
		    global.setBrushOffset(global.getBrushOffset() - 1.0);
	    }

	    g.setColor(Color.cyan);
	    int nextCycle;
	    for (int i = 0; i <= global.getVisibleCycles(); i++) {
		nextCycle = (int)Math.rint(((double)i - cycleOffset + 
		    global.getBrushOffset()) * cycleWidth);
		if (nextCycle < 0)
		    continue;
		g.fillRect(bounds.x + nextCycle + 
		    halfGlyphSizeInt - (int)Math.rint(halfGlyphSize * 
		    (greatestValue - leastValue)), 
		    bounds.y + 1, 
		    2 * (int)Math.rint(halfGlyphSize * 
		    (greatestValue - leastValue)),
		    bounds.height - 2);
	    }
	}

	// Draw guidelines (cycle markers, plot calibrations)
	if (global.getShowGuidelines()) {
	    g.setColor(Color.pink);
	    int nextCycle;
	    for (int i = 0; i <= global.getVisibleCycles(); i++) {
		nextCycle = (int)Math.rint(((double)i - cycleOffset) * cycleWidth);
		if (nextCycle < 0)
		    continue;
		g.drawLine(bounds.x + halfGlyphSizeInt + nextCycle, 
		    bounds.y + 1, 
		    bounds.x + halfGlyphSizeInt + nextCycle, 
		    bounds.y + bounds.height - 1);
	    }

	    if (optionFlags[0]) {
		g.setColor(Color.darkGray);
		g.drawLine(bounds.x + 1, plotOffset,
		    bounds.x + bounds.width - 1, plotOffset);
		g.drawLine(bounds.x + 1, plotOffset + plotScale,
		    bounds.x + bounds.width - 1, plotOffset + plotScale);

		g.setColor(Color.lightGray);
		g.drawLine(bounds.x + 1, plotOffset + (plotScale / 4),
		    bounds.x + bounds.width - 1, plotOffset + (plotScale / 4));
		g.drawLine(bounds.x + 1, plotOffset + (plotScale / 2),
		    bounds.x + bounds.width - 1, plotOffset + (plotScale / 2));
		g.drawLine(bounds.x + 1, plotOffset + (plotScale * 3 / 4),
		    bounds.x + bounds.width - 1, plotOffset + (plotScale * 3 / 4));
	    } else
		g.setColor(Color.lightGray);

	    g.drawLine(bounds.x + 1, yOrigin, 
		bounds.x + bounds.width - 1, yOrigin);
	}

	prevDataPoint = null;
	for (int i = 0; i < global.getNumDataPoints(); i++) {
	    dataPoint = global.getNormData(i);
	    tVal = dataPoint[global.getDimMap(0)];

	    // Ignore data that is outside of visible interval
	    if ((tVal < global.getTVal()) ||
		(tVal > global.getTVal2())) 
	    {
		continue;
	    }

	    // Determine glyph origin (position for center of glyph)
	    glyphX = bounds.x + 
		(int)Math.rint(halfGlyphSize + 
		(((double)bounds.width - (halfGlyphSize * 2.0)) * 
		((tVal - global.getTVal()) / global.getTInterval())));

	    if (optionFlags[0]) {
		// Draw the plot points here!

		for (int j = 0; j < global.getEnabledDim(); j++) {
		    g.setColor(Color.getHSBColor((float)j / 
			(float)global.getEnabledDim(), 
                        (float)1.0, (float)0.8));

		    if (optionFlags[1] && (prevDataPoint != null)) {
			// Draw lines between data points
			g.drawLine(prevGlyphX, plotOffset + 
			    (int)Math.rint((float)plotScale * (1.0 - 
			    ((prevDataPoint[global.getDimMap(j + 1)] - leastValue) * 
			    (1.0 / (greatestValue - leastValue))))), glyphX, 
			    plotOffset + (int)Math.rint((float)plotScale * (1.0 - 
			    ((dataPoint[global.getDimMap(j + 1)] - leastValue) * 
			    (1.0 / (greatestValue - leastValue))))));
		    }

		    g.fillRect(glyphX - 2, plotOffset - 2 + 
			(int)Math.rint((float)plotScale * (1.0 - 
			((dataPoint[global.getDimMap(j + 1)] - leastValue) * 
			(1.0 / (greatestValue - leastValue))))), 
			5, 5);
		}

		if (optionFlags[1]) {
		    prevGlyphX = glyphX;
		    prevDataPoint = dataPoint;
		}
	    }

	    if (global.getColorPerGlyph()) {
		// Determine glyph color based on cycle offset in data space
		cycleOffset = Math.IEEEremainder((tVal - global.getTMin()), 
		    global.getCycleLength());
		if (cycleOffset < 0.0)
		    cycleOffset += global.getCycleLength();
		g.setColor(Color.getHSBColor((float)(cycleOffset / 
		    global.getCycleLength()), 
                    (float)1.0, (float)0.8));
	    }

	    // Draw the apropriate glyph
	    currentGlyphType.drawGlyph(g, glyphX, yOrigin, halfGlyphSize, 
		leastValue, greatestValue, dataPoint, global);
	}

	// Draw data labels if necessary
	if (global.getShowLabels()) {
	    g.setFont(labelFont);
	    g.setColor(Color.black);
	    int labelHeight = labelFM.getAscent();
	    int lowestPoint, thisY;
	    int startCheck;
	    int cycleNumber, prevCycleNumber = -1;
	    double tValLast = -1.0;
	    boolean firstDrawn = true;
	    boolean thisExtraLowToggle = false;
	    boolean extraLowToggle = false;

	    for (int i = 0; i < global.getNumDataPoints(); i++) {
		dataPoint = global.getNormData(i);
		tVal = dataPoint[global.getDimMap(0)];

		// Ignore data that is outside of visible interval
		if ((tVal < global.getTVal()) ||
		    (tVal > global.getTVal2()))
		    continue;

		if (cycleWidth < 300) {
		    thisExtraLowToggle = false;

		    cycleNumber = (int)Math.floor((tVal - 
			global.getTMin()) / 
			global.getCycleLength());

		    if (firstDrawn) {
			firstDrawn = false;

			cycleOffset = Math.IEEEremainder(tVal - 
			    global.getTMin(), 
			    global.getCycleLength());
			if (cycleOffset < 0.0)
			    cycleOffset += global.getCycleLength();
			cycleOffset /= global.getCycleLength();

			prevCycleNumber = cycleNumber;

			if (cycleOffset != 0.0) {
			    continue;
			}
		    } else {
			if (cycleNumber != prevCycleNumber) {
			    prevCycleNumber = cycleNumber;
			} else
			    continue;
		    }
		} else {
		    if (firstDrawn) {
			firstDrawn = false;
			if (tVal != tValLast) {
			    tValLast = tVal;
			    extraLowToggle = !extraLowToggle;
			}
			thisExtraLowToggle = extraLowToggle;
		    }
		}

		// Determine glyph origin (position for center of glyph)
		glyphX = bounds.x + 
		    (int)Math.rint(halfGlyphSize + 
		    (((double)bounds.width - (halfGlyphSize * 2.0)) * 
		    ((tVal - global.getTVal()) / global.getTInterval())));

		// Get lower radius of data point
		lowestPoint = currentGlyphType.getLabelOffset(halfGlyphSize, leastValue, 
		    greatestValue, dataPoint, global);

		if (thisExtraLowToggle) {
		    lowestPoint -= (labelHeight + 1);
		}
		thisExtraLowToggle = !thisExtraLowToggle;

		// Draw the label
		g.drawString("" + tVal, 
		    glyphX - (labelFM.stringWidth("" + tVal) / 2),
		    yOrigin - lowestPoint + labelHeight);
	    }
	}

	// If the cursor is hovering over a glyph,
	// dump its raw values...
	if (global.getCursorX() != -1) {
	    if (((global.getCursorY() <= (yOrigin + (halfGlyphSizeInt / 2))) &&
		(global.getCursorY() >= (yOrigin - (halfGlyphSizeInt / 2)))) ||
		(optionFlags[0] && ((global.getCursorY() <= (plotOffset + plotScale)) && 
		(global.getCursorY() >= plotOffset))))
	    {
		for (int i = 0; i < global.getNumDataPoints(); i++) {
		    dataPoint = global.getNormData(i);
		    tVal = dataPoint[global.getDimMap(0)];

		    // Ignore data that is outside of visible interval
		    if ((tVal < global.getTVal()) ||
			(tVal > global.getTVal2()))
			continue;

		    // Determine glyph origin (position for center of glyph)
		    glyphX = bounds.x + 
			(int)Math.rint(halfGlyphSize + 
			(((double)bounds.width - (halfGlyphSize * 2.0)) * 
			((tVal - global.getTVal()) / global.getTInterval())));

		    if ((global.getCursorX() < (glyphX + (halfGlyphSizeInt / 2))) &&
			(global.getCursorX() > (glyphX - (halfGlyphSizeInt / 2))))
		    {
			int dumpAtX = global.getCursorX() + (halfGlyphSizeInt / 4) + 10;
			int dumpAtY = global.getCursorY() + (halfGlyphSizeInt / 4) + 10;

			drawDataDump(g, dumpAtX, dumpAtY, bounds.x + bounds.width, 
			    bounds.y + bounds.height, i, global);
			break;
		    }
		}
	    }
	}
    }
}