//
// StackedLayoutPlugin.java
//
// This file provides the code used to extend the DataLayout class to
// plot the data on stacked cycles.
//
package spg.plugins;

import spg.*;
import java.awt.*;

public class StackedLayoutPlugin extends DataLayout {
    private int halfGlyphSizeInt;
    private int thisY, glyphX, glyphY;
    private double halfGlyphSize;
    private double cycleOffset, cycleNumber, cycleHeight, halfCycleHeight;
    private double tVal, dataPoint[];

    public StackedLayoutPlugin() {
	super("Stacked Layout");

	halfGlyphSizeInt = thisY = glyphX = glyphY = 0;
	cycleOffset = cycleNumber = cycleHeight = halfCycleHeight = 0.0;
	halfGlyphSize = tVal = 0.0;
	dataPoint = null;
    }

    public Rectangle getLayoutBounds(Dimension d) {
	bounds.height = d.height - 20;
	bounds.y = 10;
	if (d.height > d.width) {
	   bounds.width = d.width - 20;
	   bounds.x = 10;
	} else {
	   bounds.width = bounds.height;
	   bounds.x = (d.width - bounds.width) / 2;
	}

	return bounds;
    }

    public void drawLayout(Graphics g, ViewerState global) {
	currentGlyphType = global.getGlyphPlugin();

	// Calculate cycle properties
	cycleOffset = Math.IEEEremainder(global.getTVal() - 
	    global.getTMin(), global.getCycleLength());
	if (cycleOffset < 0.0)
	    cycleOffset += global.getCycleLength();
	cycleOffset /= global.getCycleLength();
	cycleHeight = (double)bounds.height / 
	    (double)global.getVisibleCycles();
	halfCycleHeight = cycleHeight * 0.5;

	// Determine how big glyphs should be
	halfGlyphSize = (cycleHeight - 10.0) * 0.5;
	if (halfGlyphSize > ((double)(bounds.width - 10) * 0.04))
	    halfGlyphSize = (double)(bounds.width - 10) * 0.04;
	halfGlyphSize *= (global.getGlyphScale() / 
	    (greatestValue - leastValue));
	halfGlyphSizeInt = (int)Math.rint(halfGlyphSize);
	
	// Draw guidelines (cycle lines and markers)
	if (global.getShowGuidelines()) {
	    g.setColor(Color.lightGray);
	    for (int i = 1; i <= global.getVisibleCycles(); i++) {
		thisY = (int)Math.rint(((double)i * cycleHeight) - 
		    halfCycleHeight);
		g.drawLine(bounds.x + 1, bounds.y + thisY,
		    bounds.x + bounds.width - 1, bounds.y + thisY);
	    }

	    g.setColor(Color.pink);
	    int nextCycle;
	    for (int i = 0; i < 2; i++) {
		nextCycle = (int)Math.rint(((double)i - cycleOffset) * 
		    ((double)bounds.width - (halfGlyphSize * 2.0)));
		if (nextCycle < 0)
		    continue;
		g.drawLine(bounds.x + halfGlyphSizeInt + nextCycle, 
		    bounds.y + 1, 
		    bounds.x + halfGlyphSizeInt + nextCycle, 
		    bounds.y + bounds.height - 1);
	    }
	}

	for (int i = 0; i < global.getNumDataPoints(); i++) {
	    dataPoint = global.getNormData(i);
	    tVal = dataPoint[global.getDimMap(0)];

	    // Ignore data that is outside of visible interval
	    if ((tVal < global.getTVal()) ||
		(tVal > global.getTVal2()))
		  continue;

	    // Determine glyph origin (position for center of glyph)
	    cycleOffset = Math.IEEEremainder((tVal - global.getTVal()),
 		global.getCycleLength());
	    if (cycleOffset < 0.0)
		cycleOffset += global.getCycleLength();
	    cycleOffset /= global.getCycleLength();
	    cycleNumber = Math.floor((tVal - global.getTVal()) / 
		global.getCycleLength()) + 1.0;
	    glyphY = bounds.y + (int)Math.rint((cycleNumber * cycleHeight) - 
		halfCycleHeight);
	    glyphX = bounds.x + 
		(int)Math.rint(halfGlyphSize + 
		((double)bounds.width - (halfGlyphSize * 2.0)) * cycleOffset);
	    cycleOffset = Math.IEEEremainder((tVal - global.getTMin()),
 		global.getCycleLength());
	    if (cycleOffset < 0.0)
		cycleOffset += global.getCycleLength();
	    cycleOffset /= global.getCycleLength();

	    if (global.getColorPerGlyph()) {
		// Determine glyph color based on cycle offset in data space
		cycleOffset = Math.IEEEremainder((tVal - global.getTMin()),
 		    global.getCycleLength());
		if (cycleOffset < 0.0)
		    cycleOffset += global.getCycleLength();
		cycleOffset /= global.getCycleLength();
		g.setColor(Color.getHSBColor(
		    (float)cycleOffset, (float)1.0, (float)0.8));
	    }

	    // Draw the apropriate glyph
	    currentGlyphType.drawGlyph(g, glyphX, glyphY, halfGlyphSize, 
		leastValue, greatestValue, dataPoint, global);
	}

	// Draw data labels if necessary
	if (global.getShowLabels()) {
	    g.setFont(labelFont);
	    g.setColor(Color.black);
	    int labelHeight = labelFM.getAscent();
	    int lowestPoint;
	    int startCheck;

	    for (int i = 0; i < global.getNumDataPoints(); i++) {
		dataPoint = global.getNormData(i);
		tVal = dataPoint[global.getDimMap(0)];

		// Ignore data that is outside of visible interval
		if ((tVal < global.getTVal()) ||
		    (tVal > global.getTVal2()))
		    continue;

		// Determine glyph origin (position for center of glyph)
		cycleOffset = Math.IEEEremainder((tVal - global.getTVal()),
 		    global.getCycleLength());
		if (cycleOffset < 0.0)
		    cycleOffset += global.getCycleLength();
		cycleOffset /= global.getCycleLength();
		cycleNumber = Math.floor((tVal - global.getTVal()) / 
		    global.getCycleLength()) + 1.0;
		glyphY = bounds.y + (int)Math.rint((cycleNumber * cycleHeight) - 
		    halfCycleHeight);
		glyphX = bounds.x + 
		    (int)Math.rint(halfGlyphSize + 
		    ((double)bounds.width - (halfGlyphSize * 2.0)) * cycleOffset);

		// Get lower radius of data point
		lowestPoint = currentGlyphType.getLabelOffset(halfGlyphSize, leastValue, 
		    greatestValue, dataPoint, global);

		// Draw the label
		g.drawString("" + tVal, 
		    glyphX - (labelFM.stringWidth("" + tVal) / 2),
		    glyphY - lowestPoint + labelHeight);
	    }
	}

	// If the cursor is hovering over a glyph,
	// dump its raw values...
	if (global.getCursorX() != -1) {
	    for (int i = 0; i < global.getNumDataPoints(); i++) {
		dataPoint = global.getNormData(i);
		tVal = dataPoint[global.getDimMap(0)];

		// Ignore data that is outside of visible interval
		if ((tVal < global.getTVal()) ||
		    (tVal > global.getTVal2()))
		    continue;

		// Determine glyph origin (position for center of glyph)
		cycleOffset = Math.IEEEremainder((tVal - global.getTVal()),
 		    global.getCycleLength());
		if (cycleOffset < 0.0)
		    cycleOffset += global.getCycleLength();
		cycleOffset /= global.getCycleLength();
		cycleNumber = Math.floor((tVal - global.getTVal()) / 
		    global.getCycleLength()) + 1.0;
		glyphY = bounds.y + (int)Math.rint((cycleNumber * cycleHeight) - 
		    halfCycleHeight);
		glyphX = bounds.x + 
		    (int)Math.rint(halfGlyphSize + 
		    ((double)bounds.width - (halfGlyphSize * 2.0)) * cycleOffset);

		if ((global.getCursorX() < (glyphX + (halfGlyphSizeInt / 2))) &&
		    (global.getCursorX() > (glyphX - (halfGlyphSizeInt / 2))) &&
		    (global.getCursorY() < (glyphY + (halfGlyphSizeInt / 2))) &&
		    (global.getCursorY() > (glyphY - (halfGlyphSizeInt / 2))))
		{
		    int dumpAtX = global.getCursorX() + (halfGlyphSizeInt / 4) + 10;
		    int dumpAtY = global.getCursorY() + (halfGlyphSizeInt / 4) + 10;

		    drawDataDump(g, dumpAtX, dumpAtY, bounds.x + bounds.width, 
			bounds.y + bounds.height, i, global);
		    break;
		}
	    }
	}
    }
}
